==> fxt-2006.12.17.tgz: fxt/src/comb/setpartition-rgs.h <==
#if !defined  HAVE_SETPARTITION_RGS_H__
#define       HAVE_SETPARTITION_RGS_H__

#include "sort/minmaxmed23.h"  // max2()

#include "fxttypes.h"


#define SETPART_RGS_ITERATIVE

class set_partition_rgs
// Set partitions of the set {1,2,3,...,n}
//   as restricted growth strings (RGS).
// By default in minimal-change order for set partitions,
//  the RGS can change in more than one position.
{
public:
    ulong n_;   // Number of elements of set (set = {1,2,3,...,n})
    int *ns_;   // ns[k] Number of Sets in set partition k
    int *as_;   // element k attached At Set (0<=as[k]<=k) of set(k-1)
    int *d_;    // direction with recursion (+1 or -1)
    bool xdr_;  // whether to change direction in recursion (==> minimal-change order)
    int dr0_;   // dr0: starting direction in each recursive step:
    //   dr0=+1  ==> start with partition  {{1,2,3,...,n}}
    //   dr0=-1  ==> start with partition  {{1},{2},{3},...,{n}}}

public:
    set_partition_rgs(ulong n, bool xdr=true, int dr0=+1)
        : n_(n)
    {
        ns_ = new int[n_+1];
        as_ = new int[n_+1];
        d_ = new int[n_+1];
        init(xdr, dr0);
    }

    ~set_partition_rgs()
    {
        delete [] ns_;
        delete [] as_;
        delete [] d_;
    }

    void init(bool xdr, int dr0);

    bool next()
#ifdef SETPART_RGS_ITERATIVE
    {
        ulong k = n_;
        while ( (ulong)(as_[k] + d_[k]) > (ulong)ns_[k-1] ) // <0 or >max
        {
            if ( --k <= 1 )  return 0;
        }

        as_[k] += d_[k];
        ns_[k] = max2(ns_[k-1], as_[k]+1);

        while ( ++k<=n_ )
        {
            ulong d = d_[k];
            d = ( xdr_ ? -d : dr0_ );
            d_[k] = d;
            ulong as = ( ((int)d>0) ? 0 : ns_[k-1] );
            as_[k] = as;
            ns_[k] = max2(ns_[k-1], as_[k]+1);
        }

        return 1;
    }
#else // SETPART_RGS_ITERATIVE
    {
        return next_rec(n_);
    }
#endif // SETPART_RGS_ITERATIVE

    const int* data()  const
    { return as_; }

    void print()  const;

protected:
    int next_rec(ulong k);
};
// -------------------------


#endif  // !defined HAVE_SETPARTITION_RGS_H__

==> fxt-2008.02.02.tgz: fxt/src/comb/setpartition-rgs.h <==
#if !defined  HAVE_SETPARTITION_RGS_H__
#define       HAVE_SETPARTITION_RGS_H__

#include "sort/minmaxmed23.h"  // max2()

#include "fxttypes.h"


#define SETPART_RGS_ITERATIVE

class set_partition_rgs
// Set partitions of the set {1,2,3,...,n}
//   as restricted growth strings (RGS).
// By default in minimal-change order for set partitions,
//  the RGS can change in more than one position.
{
public:
    ulong n_;   // Number of elements of set (set = {1,2,3,...,n})
    int *ns_;   // ns[k] Number of Sets in set partition k
    int *as_;   // element k attached At Set (0<=as[k]<=k) of set(k-1)
    int *d_;    // direction with recursion (+1 or -1)
    bool xdr_;  // whether to change direction in recursion (==> minimal-change order)
    int dr0_;   // dr0: starting direction in each recursive step:
    //   dr0=+1  ==> start with partition  {{1,2,3,...,n}}
    //   dr0=-1  ==> start with partition  {{1},{2},{3},...,{n}}}

public:
    set_partition_rgs(ulong n, bool xdr=true, int dr0=+1)
        : n_(n)
    {
        ns_ = new int[n_+1];
        as_ = new int[n_+1];
        d_ = new int[n_+1];
        init(xdr, dr0);
    }

    ~set_partition_rgs()
    {
        delete [] ns_;
        delete [] as_;
        delete [] d_;
    }

    void init(bool xdr, int dr0);

    bool next()
#ifdef SETPART_RGS_ITERATIVE
    {
        ulong k = n_;
        while ( (ulong)(as_[k] + d_[k]) > (ulong)ns_[k-1] ) // <0 or >max
        {
            if ( --k <= 1 )  return false;
        }

        as_[k] += d_[k];
        ns_[k] = max2(ns_[k-1], as_[k]+1);

        while ( ++k<=n_ )
        {
            ulong d = d_[k];
            d = ( xdr_ ? -d : dr0_ );
            d_[k] = d;
            ulong as = ( ((int)d>0) ? 0 : ns_[k-1] );
            as_[k] = as;
            ns_[k] = max2(ns_[k-1], as_[k]+1);
        }

        return true;
    }
#else // SETPART_RGS_ITERATIVE
    {
        return next_rec(n_);
    }
#endif // SETPART_RGS_ITERATIVE

    const int* data()  const  { return as_; }

    void print()  const;

protected:
    int next_rec(ulong k);
};
// -------------------------


#endif  // !defined HAVE_SETPARTITION_RGS_H__
